/*
 *                 IFS Research & Development
 *
 *  This program is protected by copyright law and by international
 *  conventions. All licensing, renting, lending or copying (including
 *  for private use), and all other use of the program, which is not
 *  expressively permitted by IFS Research & Development (IFS), is a
 *  violation of the rights of IFS. Such violations will be reported to the
 *  appropriate authorities.
 *
 *  VIOLATIONS OF ANY COPYRIGHT IS PUNISHABLE BY LAW AND CAN LEAD
 *  TO UP TO TWO YEARS OF IMPRISONMENT AND LIABILITY TO PAY DAMAGES.
 * ----------------------------------------------------------------------------
 * File        : MyLoopConnectReader.java
 * Description :
 * Notes       :
 * ----------------------------------------------------------------------------
 * Modified    :
 *    2016-Feb-18  Jacek P  Created.
 * ----------------------------------------------------------------------------
 */

package ifs.fndint.connectreader;

import ifs.fnd.connect.readers.NolistConnectReader;
import ifs.fnd.util.IoUtil;
import java.io.File;
import java.io.FileFilter;
import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.Map;
import java.util.SortedMap;
import java.util.TreeMap;

public class MyLoopConnectReader extends NolistConnectReader<MyLoopConnectReaderConfig> {

   /**
    * Initialization of resources used by the reader. The reader implements the
    * AutoCloseable interface, so all resources initiated/open here have to be
    * released/closed in the nativeClose() method.
    * @throws ReaderFailureException 
    */
   @Override
   protected void nativeInit() throws ReaderFailureException {
      if(log.debug) log.debug("Initializating My List Connect Reader [&1]...", config.instance);
      // create IN directory if no already exists
      try {
         IoUtil.mkdirs(new File(config.inDir));
      }
      catch(IOException e) {
         throw new PermanentFailureException(e, "Error while creating input directory [&1]", config.inDir);
      }
   }

   @Override
   protected void nativeLoop() throws ReaderFailureException {
      File inDir = new File(config.inDir);
      File[] files = inDir.listFiles(new FileFilter(){
         @Override
         public boolean accept(File file) {
            if(file==null || !file.isFile()) {
               if(log.debug) log.debug("File [&1] is null or not a file", file);
               return false;
            }
            boolean doAccept = selectMessage(file.getName());
            if(log.debug) log.debug("File [&1] &2satisfies filter", file, doAccept?"":"NOT ");
            return doAccept;
         }
      });
      try {
         if(log.debug) log.debug("Reading content of directory [&1]...", inDir.getCanonicalPath());
         SortedMap<String,File> sortedFiles = new TreeMap<>();
         for(File file:files) {
            sortedFiles.put(file.getName(), file);
         }
         for(Map.Entry<String,File> entry:sortedFiles.entrySet()) {
            String fileName = entry.getKey();
            File file = entry.getValue();
            if(log.debug) log.debug("Found file [&1]; adding to message list...", fileName);
            Path path = Paths.get(file.toURI());
            Message msg = new Message(fileName, fileName);
            msg.setData(Files.readAllBytes(path));
            processMessage(msg);
            IoUtil.delete(file);
         }
      }
      catch(IOException e) {
         throw new TemporaryFailureException(e, "Error while processing directory [&1]", config.inDir);
      }
   }

   @Override
   public void nativeClose() throws PermanentFailureException {
      // close all resurces open by nativeInit() here
   }
}
